#!/bin/bash
TMP=output.tmp
RECENT=emoji_recent
EMOJIS=emojis
CURRENTFILE=$EMOJIS
MAXRECENT=50


_grep() {
  if [[ $wholewords -eq 1 ]] 
  then
    grep -w -- "$@" 
  else
    grep -- "$@"
  fi
}

addrecent() {
  echo "$1" >> $RECENT
  tail -r -${MAXRECENT} $RECENT | awk '!x[$0]++' | tail -r -${MAXRECENT} > $RECENT.tmp
  mv $RECENT.tmp $RECENT
}

#filter results term by term
multigrep() {
  local grepline ARG
  ARG=$(echo "$1" | tr ' ' '\n')
  for i in $ARG
  do
    grepline="$grepline | _grep \"$i\"" 
  done

  grepline="${grepline/|/}"
  cat "$CURRENTFILE" | eval "$grepline"
}

querydb() {
  local query="$1"
  if [[ -z $query && $isrecent -eq 1 ]] 
  then
    emojisfound=$(tail -r -${MAXRECENT} $CURRENTFILE)
  else
    if isemojiquery "$query" 
    then
      EMOJIQUERY=1
      query="${query#* }"
    fi
    #check if multiple terms
    if [[ $query =~ " " ]]
    then
      emojisfound=$(multigrep "$query")
    else
      emojisfound=$(_grep "$query" $CURRENTFILE)
    fi
  fi
}

addemojigroup() {
  local ARG CATS EMO
  ARG="$1"
  [[ "$ARG" =~ "argaddgroup" ]] && ARG="${ARG/argaddgroup /}"
  # ARG=${ARG/|/,}
  EMO="${ARG/|*/}"
  CATS="${ARG/*|/}"
  grep -q "^$EMO," "$EMOJIS" && { echo "Error: cannot add new group, group $EMO exists" ; exit 1 ; }
  echo "$EMO,[G] $CATS" >> $EMOJIS
  echo "Added $EMO as a new group with ${CATS// /, } as search terms"
}

addcat() {
  local EMO CATS ARG LINE matches

  ARG="$1"
  [[ "$ARG" =~ "argaddcat" ]] && ARG="${ARG/argaddcat /}"
  
  EMO="${ARG/|*/}"
  CATS="${ARG/*|/}"
  matches=$(grep "^$EMO," $CURRENTFILE | wc -l) 
  [[ $matches -ne 1 ]] && echo "Error: Unexpected matching groups found(${matches// /}) for:$EMO" && exit 1
  
  echo "${EMO} - Adding categories"
  #check for existing categories
  LINE=$(grep "^$EMO," $CURRENTFILE)
  for cat in $(echo $CATS)
  do
    if [[ ! "$LINE " =~ " $cat " ]] 
    then
      [[ -z $TOADD ]] && TOADD="$cat" || TOADD="${TOADD} $cat"
    else
      echo "$cat already exists, skipping"
    fi
  done

  [[ -n "$TOADD" ]] && sed -i .bak "/$EMO/ s/$/ $TOADD/" $CURRENTFILE &&
    echo "$TOADD added succesfuly"
  #rm $CURRENTFILE.bak
}

isemojiquery() {
  local QUERY="$1" EMOJI
  EMOJI=${QUERY:0:1}
  grep -q "^$EMOJI" "$CURRENTFILE"
}

noresults() {
cat <<EOF
      {
        "valid": false,
        "title": "No terms match your query",
        "subtitle" : "Try another term",
        "autocomplete" : ""
      }
EOF
}

jsonaddgroup() {

  local currentemojis="$1" argjson
  # Set first result as option to add new terms
  if [[ ! "$query" =~ [a-zA-Z] ]] 

  then
    #remove trailing space
    termstoadd="${termstoadd/% /}"
    #Remove Some space are created by emoji codes
    currentemojis=${currentemojis// /}
    if [[ $EMOJIQUERY -eq 1 && ${#currentemojis} -gt 1 && ! $(grep  "${currentemojis}" $EMOJIS) ]]
    then
      argjson="\"arg\":\"argaddgroup $currentemojis|$termstoadd\","
    else
      argjson="\"arg\":\"argaddcat $query|$termstoadd\","
    fi
    cat <<EOF >> $TMP
      {
        "valid": true,
        "title": "Add ${termstoadd// /, } to group:  $query",
        $argjson
        "autocomplete": "$currentemojis",
        "subtitle" : "Delimit additional groups with space"
      },
EOF
  fi
}

buildjson() {
  local counter delimitgroups emoji arr currentemojis argjson subtitlesCurrentEmojis lastitem title
  #build json
  echo -E "{\"items\": [" > $TMP

  #Is the search is an emoji(s)
  if [[ $EMOJIQUERY -eq 1 ]]
  then
    currentemojis=${query/ */}
    currentemojis=${currentemojis// /}
  fi
  
  [[ -n $termstoadd ]] && jsonaddgroup "$currentemojis"

  if [[ -z "$emojisfound" && $EMOJIQUERY -ne 1 ]]
  then
    noresults >> $TMP
  else
    #Output all emojis found as Json
    declare -a arr
    counter=0
    while read -r item
    do
      counter=$((counter+1))
      # echo -n $counter
      if [[ ${isrecent:-0} -eq 0 ]]
      then
        emoji=${item/,*/}
        desc="${item/*,/}"
        title="${emoji} ${desc/ */}"
      else
        emoji=${item}
        desc=""
        title="${emoji}"
      fi
      if [[ $EMOJIQUERY -eq 1 ]]
      then
        # Add the current emojis to autocomplete and add subtitles
        subtitlesCurrentEmojis=", \"subtitle\" : \"Que:${currentemojis} ${desc}\""
        arr+=("{ \"valid\": true, \"title\": \"${title}\", \"arg\": \"${currentemojis}\", \"autocomplete\": \"$currentemojis${emoji}\" $subtitlesCurrentEmojis },")
      else
        arr+=("{ \"valid\": true, \"title\": \"${title}\", \"arg\": \"$emoji\", \"autocomplete\": \"$emoji\", \"subtitle\" : \"$desc\"  },")
      fi
    done <<<"$emojisfound"
    #finish Json
    lastitem=$((${#arr[@]}-1))
    arr[$lastitem]="${arr[$lastitem]/%,/}"
    echo "${arr[*]}" >> $TMP
  fi
  echo -E "]}" >> $TMP
}

#Main 
[[ $1 == "argaddgroup" ]] && addemojigroup "$2" && exit 0
[[ $1 == "argaddrecent" ]] && addrecent "$2" && exit 0
[[ $1 == "argaddcat" ]] && { addcat "$2" ; exit 0 ; }
[[ $1 == "argrecent" ]] && shift && CURRENTFILE=$RECENT && isrecent=1
[[ $1 == "argwhole" ]] && wholewords=1 && shift

emojisfound=""
query="$1"
[[ $query =~ "|" ]] && termstoadd=${query/*|/} && query="${query/|*/}"

querydb "$query"
buildjson

#output
cat $TMP

#cleanup
rm $TMP

